/*
 *                                  Apache License
 *                            Version 2.0, January 2004
 *                         http://www.apache.org/licenses/
 *
 *    TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION
 *
 *    1. Definitions.
 *
 *       "License" shall mean the terms and conditions for use, reproduction,
 *       and distribution as defined by Sections 1 through 9 of this document.
 *
 *       "Licensor" shall mean the copyright owner or entity authorized by
 *       the copyright owner that is granting the License.
 *
 *       "Legal Entity" shall mean the union of the acting entity and all
 *       other entities that control, are controlled by, or are under common
 *       control with that entity. For the purposes of this definition,
 *       "control" means (i) the power, direct or indirect, to cause the
 *       direction or management of such entity, whether by contract or
 *       otherwise, or (ii) ownership of fifty percent (50%) or more of the
 *       outstanding shares, or (iii) beneficial ownership of such entity.
 *
 *       "You" (or "Your") shall mean an individual or Legal Entity
 *       exercising permissions granted by this License.
 *
 *       "Source" form shall mean the preferred form for making modifications,
 *       including but not limited to software source code, documentation
 *       source, and configuration files.
 *
 *       "Object" form shall mean any form resulting from mechanical
 *       transformation or translation of a Source form, including but
 *       not limited to compiled object code, generated documentation,
 *       and conversions to other media types.
 *
 *       "Work" shall mean the work of authorship, whether in Source or
 *       Object form, made available under the License, as indicated by a
 *       copyright notice that is included in or attached to the work
 *       (an example is provided in the Appendix below).
 *
 *       "Derivative Works" shall mean any work, whether in Source or Object
 *       form, that is based on (or derived from) the Work and for which the
 *       editorial revisions, annotations, elaborations, or other modifications
 *       represent, as a whole, an original work of authorship. For the purposes
 *       of this License, Derivative Works shall not include works that remain
 *       separable from, or merely link (or bind by name) to the interfaces of,
 *       the Work and Derivative Works thereof.
 *
 *       "Contribution" shall mean any work of authorship, including
 *       the original version of the Work and any modifications or additions
 *       to that Work or Derivative Works thereof, that is intentionally
 *       submitted to Licensor for inclusion in the Work by the copyright owner
 *       or by an individual or Legal Entity authorized to submit on behalf of
 *       the copyright owner. For the purposes of this definition, "submitted"
 *       means any form of electronic, verbal, or written communication sent
 *       to the Licensor or its representatives, including but not limited to
 *       communication on electronic mailing lists, source code control systems,
 *       and issue tracking systems that are managed by, or on behalf of, the
 *       Licensor for the purpose of discussing and improving the Work, but
 *       excluding communication that is conspicuously marked or otherwise
 *       designated in writing by the copyright owner as "Not a Contribution."
 *
 *       "Contributor" shall mean Licensor and any individual or Legal Entity
 *       on behalf of whom a Contribution has been received by Licensor and
 *       subsequently incorporated within the Work.
 *
 *    2. Grant of Copyright License. Subject to the terms and conditions of
 *       this License, each Contributor hereby grants to You a perpetual,
 *       worldwide, non-exclusive, no-charge, royalty-free, irrevocable
 *       copyright license to reproduce, prepare Derivative Works of,
 *       publicly display, publicly perform, sublicense, and distribute the
 *       Work and such Derivative Works in Source or Object form.
 *
 *    3. Grant of Patent License. Subject to the terms and conditions of
 *       this License, each Contributor hereby grants to You a perpetual,
 *       worldwide, non-exclusive, no-charge, royalty-free, irrevocable
 *       (except as stated in this section) patent license to make, have made,
 *       use, offer to sell, sell, import, and otherwise transfer the Work,
 *       where such license applies only to those patent claims licensable
 *       by such Contributor that are necessarily infringed by their
 *       Contribution(s) alone or by combination of their Contribution(s)
 *       with the Work to which such Contribution(s) was submitted. If You
 *       institute patent litigation against any entity (including a
 *       cross-claim or counterclaim in a lawsuit) alleging that the Work
 *       or a Contribution incorporated within the Work constitutes direct
 *       or contributory patent infringement, then any patent licenses
 *       granted to You under this License for that Work shall terminate
 *       as of the date such litigation is filed.
 *
 *    4. Redistribution. You may reproduce and distribute copies of the
 *       Work or Derivative Works thereof in any medium, with or without
 *       modifications, and in Source or Object form, provided that You
 *       meet the following conditions:
 *
 *       (a) You must give any other recipients of the Work or
 *           Derivative Works a copy of this License; and
 *
 *       (b) You must cause any modified files to carry prominent notices
 *           stating that You changed the files; and
 *
 *       (c) You must retain, in the Source form of any Derivative Works
 *           that You distribute, all copyright, patent, trademark, and
 *           attribution notices from the Source form of the Work,
 *           excluding those notices that do not pertain to any part of
 *           the Derivative Works; and
 *
 *       (d) If the Work includes a "NOTICE" text file as part of its
 *           distribution, then any Derivative Works that You distribute must
 *           include a readable copy of the attribution notices contained
 *           within such NOTICE file, excluding those notices that do not
 *           pertain to any part of the Derivative Works, in at least one
 *           of the following places: within a NOTICE text file distributed
 *           as part of the Derivative Works; within the Source form or
 *           documentation, if provided along with the Derivative Works; or,
 *           within a display generated by the Derivative Works, if and
 *           wherever such third-party notices normally appear. The contents
 *           of the NOTICE file are for informational purposes only and
 *           do not modify the License. You may add Your own attribution
 *           notices within Derivative Works that You distribute, alongside
 *           or as an addendum to the NOTICE text from the Work, provided
 *           that such additional attribution notices cannot be construed
 *           as modifying the License.
 *
 *       You may add Your own copyright statement to Your modifications and
 *       may provide additional or different license terms and conditions
 *       for use, reproduction, or distribution of Your modifications, or
 *       for any such Derivative Works as a whole, provided Your use,
 *       reproduction, and distribution of the Work otherwise complies with
 *       the conditions stated in this License.
 *
 *    5. Submission of Contributions. Unless You explicitly state otherwise,
 *       any Contribution intentionally submitted for inclusion in the Work
 *       by You to the Licensor shall be under the terms and conditions of
 *       this License, without any additional terms or conditions.
 *       Notwithstanding the above, nothing herein shall supersede or modify
 *       the terms of any separate license agreement you may have executed
 *       with Licensor regarding such Contributions.
 *
 *    6. Trademarks. This License does not grant permission to use the trade
 *       names, trademarks, service marks, or product names of the Licensor,
 *       except as required for reasonable and customary use in describing the
 *       origin of the Work and reproducing the content of the NOTICE file.
 *
 *    7. Disclaimer of Warranty. Unless required by applicable law or
 *       agreed to in writing, Licensor provides the Work (and each
 *       Contributor provides its Contributions) on an "AS IS" BASIS,
 *       WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 *       implied, including, without limitation, any warranties or conditions
 *       of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A
 *       PARTICULAR PURPOSE. You are solely responsible for determining the
 *       appropriateness of using or redistributing the Work and assume any
 *       risks associated with Your exercise of permissions under this License.
 *
 *    8. Limitation of Liability. In no event and under no legal theory,
 *       whether in tort (including negligence), contract, or otherwise,
 *       unless required by applicable law (such as deliberate and grossly
 *       negligent acts) or agreed to in writing, shall any Contributor be
 *       liable to You for damages, including any direct, indirect, special,
 *       incidental, or consequential damages of any character arising as a
 *       result of this License or out of the use or inability to use the
 *       Work (including but not limited to damages for loss of goodwill,
 *       work stoppage, computer failure or malfunction, or any and all
 *       other commercial damages or losses), even if such Contributor
 *       has been advised of the possibility of such damages.
 *
 *    9. Accepting Warranty or Additional Liability. While redistributing
 *       the Work or Derivative Works thereof, You may choose to offer,
 *       and charge a fee for, acceptance of support, warranty, indemnity,
 *       or other liability obligations and/or rights consistent with this
 *       License. However, in accepting such obligations, You may act only
 *       on Your own behalf and on Your sole responsibility, not on behalf
 *       of any other Contributor, and only if You agree to indemnify,
 *       defend, and hold each Contributor harmless for any liability
 *       incurred by, or claims asserted against, such Contributor by reason
 *       of your accepting any such warranty or additional liability.
 *
 *    END OF TERMS AND CONDITIONS
 *
 *    APPENDIX: How to apply the Apache License to your work.
 *
 *       To apply the Apache License to your work, attach the following
 *       boilerplate notice, with the fields enclosed by brackets "{}"
 *       replaced with your own identifying information. (Don't include
 *       the brackets!)  The text should be enclosed in the appropriate
 *       comment syntax for the file format. We also recommend that a
 *       file or class name and description of purpose be included on the
 *       same "printed page" as the copyright notice for easier
 *       identification within third-party archives.
 *
 *    Copyright {yyyy} {name of copyright owner}
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.mineacademy.fo.jsonsimple;

import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.util.List;
import java.util.Map;
import java.util.Stack;

import org.mineacademy.fo.exception.FoException;

import lombok.Getter;

/**
 * Parses JSON data (<u>not</u> thread-safe).
 *
 * @author FangYidong(fangyidong @ yahoo.com.cn)
 * @author Ralph Niemitz/RalleYTN(ralph.niemitz@gmx.de)
 * @version 2.0.0
 * @since 1.0.0
 */
public class JSONParser {

	// ==== 11.03.2018 | Ralph Niemitz/RalleYTN(ralph.niemitz@gmx.de)
	// -	Removed the ContainerFactory
	// -	Updated the documentation
	// ====

	@Getter
	private static final JSONParser instance = new JSONParser();

	private static final int S_INIT = 0;
	private static final int S_IN_FINISHED_VALUE = 1;
	private static final int S_IN_OBJECT = 2;
	private static final int S_IN_ARRAY = 3;
	private static final int S_PASSED_PAIR_KEY = 4;
	private static final int S_IN_PAIR_VALUE = 5;
	private static final int S_END = 6;
	private static final int S_IN_ERROR = -1;

	private Stack<Object> handlerStatusStack;
	private final Yylex lexer = new Yylex((Reader) null);
	private Yytoken token;
	private int status = JSONParser.S_INIT;

	private final void nextToken() throws JSONParseException, IOException {

		this.token = this.lexer.yylex();

		if (this.token == null)
			this.token = new Yytoken(Yytoken.TYPE_EOF, null);
	}

	private final void init(Stack<Object> statusStack, Stack<Object> valueStack) {

		if (this.token.type == Yytoken.TYPE_VALUE) {

			this.status = JSONParser.S_IN_FINISHED_VALUE;
			statusStack.push(this.status);
			valueStack.push(this.token.value);

		} else if (this.token.type == Yytoken.TYPE_LEFT_BRACE) {

			this.status = JSONParser.S_IN_OBJECT;
			statusStack.push(this.status);
			valueStack.push(new JSONObject());

		} else if (this.token.type == Yytoken.TYPE_LEFT_SQUARE) {

			this.status = JSONParser.S_IN_ARRAY;
			statusStack.push(this.status);
			valueStack.push(new JSONArray());

		} else
			this.status = JSONParser.S_IN_ERROR;
	}

	private final Object inFinishedValue(Stack<Object> valueStack) throws JSONParseException {

		if (this.token.type == Yytoken.TYPE_EOF)
			return valueStack.pop();
		else
			throw new JSONParseException(this.getPosition(), JSONParseException.ERROR_UNEXPECTED_TOKEN, this.token);
	}

	private final void inObject(Stack<Object> statusStack, Stack<Object> valueStack) {

		if (this.token.type == Yytoken.TYPE_VALUE) {

			if (this.token.value instanceof String) {

				final String key = (String) this.token.value;
				valueStack.push(key);
				this.status = JSONParser.S_PASSED_PAIR_KEY;
				statusStack.push(this.status);

			} else
				this.status = S_IN_ERROR;

		} else if (this.token.type == Yytoken.TYPE_RIGHT_BRACE) {

			if (valueStack.size() > 1) {

				statusStack.pop();
				valueStack.pop();
				this.status = (int) statusStack.peek();

			} else
				this.status = JSONParser.S_IN_FINISHED_VALUE;

		} else if (this.token.type != Yytoken.TYPE_COMMA)
			this.status = JSONParser.S_IN_ERROR;
	}

	private final void inPassedPairKey(Stack<Object> statusStack, Stack<Object> valueStack) {

		if (this.token.type == Yytoken.TYPE_VALUE) {

			statusStack.pop();
			final String key = (String) valueStack.pop();
			final Map<Object, Object> parent = (Map<Object, Object>) valueStack.peek();
			parent.put(key, this.token.value);
			this.status = (int) statusStack.peek();

		} else if (this.token.type == Yytoken.TYPE_LEFT_SQUARE) {

			statusStack.pop();
			final String key = (String) valueStack.pop();
			final Map<Object, Object> parent = (Map<Object, Object>) valueStack.peek();
			final List<Object> newArray = new JSONArray();
			parent.put(key, newArray);
			this.status = JSONParser.S_IN_ARRAY;
			statusStack.push(this.status);
			valueStack.push(newArray);

		} else if (this.token.type == Yytoken.TYPE_LEFT_BRACE) {

			statusStack.pop();
			final String key = (String) valueStack.pop();
			final Map<Object, Object> parent = (Map<Object, Object>) valueStack.peek();
			final Map<Object, Object> newObject = new JSONObject();
			parent.put(key, newObject);
			this.status = JSONParser.S_IN_OBJECT;
			statusStack.push(this.status);
			valueStack.push(newObject);

		} else if (this.token.type != Yytoken.TYPE_COLON)
			this.status = JSONParser.S_IN_ERROR;
	}

	private final void inArray(Stack<Object> statusStack, Stack<Object> valueStack) {

		if (this.token.type == Yytoken.TYPE_VALUE) {

			final List<Object> val = (List<Object>) valueStack.peek();
			val.add(this.token.value);

		} else if (this.token.type == Yytoken.TYPE_RIGHT_SQUARE) {

			if (valueStack.size() > 1) {

				statusStack.pop();
				valueStack.pop();
				this.status = (int) statusStack.peek();

			} else
				this.status = JSONParser.S_IN_FINISHED_VALUE;

		} else if (this.token.type == Yytoken.TYPE_LEFT_BRACE) {

			final List<Object> val = (List<Object>) valueStack.peek();
			final Map<Object, Object> newObject = new JSONObject();
			val.add(newObject);
			this.status = JSONParser.S_IN_OBJECT;
			statusStack.push(this.status);
			valueStack.push(newObject);

		} else if (this.token.type == Yytoken.TYPE_LEFT_SQUARE) {

			final List<Object> val = (List<Object>) valueStack.peek();
			final List<Object> newArray = new JSONArray();
			val.add(newArray);
			this.status = JSONParser.S_IN_ARRAY;
			statusStack.push(this.status);
			valueStack.push(newArray);

		} else if (this.token.type != Yytoken.TYPE_COMMA)
			this.status = JSONParser.S_IN_ERROR;
	}

	/**
	 * Resets the parser to the initial state without resetting the underlying reader.
	 *
	 * @since 1.0.0
	 */
	public void reset() {

		this.token = null;
		this.status = JSONParser.S_INIT;
		this.handlerStatusStack = null;
	}

	/**
	 * Resets the parser to the initial state with a new character reader.
	 *
	 * @param reader the new character reader
	 * @since 1.0.0
	 */
	public void reset(Reader reader) {

		this.lexer.yyreset(reader);
		this.reset();
	}

	/**
	 * @return the position where the current token begins
	 * @since 1.0.0
	 */
	public int getPosition() {

		return this.lexer.getPosition();
	}

	/**
	 * Parses JSON data.
	 *
	 * @param json the JSON data
	 * @return An instance of:
	 * <ul>
	 * <li>{@linkplain JSONObject}</li>
	 * <li>{@linkplain JSONArray}</li>
	 * <li>{@linkplain String}</li>
	 * <li>{@linkplain Number}</li>
	 * <li>{@linkplain Boolean}</li>
	 * <li>{@code null}</li>
	 * </ul>
	 * @throws JSONParseException if the JSON is invalid
	 * @since 1.0.0
	 */
	public Object parse(String json) throws JSONParseException {

		try (StringReader reader = new StringReader(json)) {

			return this.parse(reader);

		} catch (final IOException exception) {

			// WILL NEVER HAPPEN!
			throw new FoException(exception);
		}
	}

	/**
	 * Parses JSON data from a {@linkplain Reader}.
	 *
	 * @param reader the {@linkplain Reader}
	 * @return An instance of:
	 * <ul>
	 * <li>{@linkplain JSONObject}</li>
	 * <li>{@linkplain JSONArray}</li>
	 * <li>{@linkplain String}</li>
	 * <li>{@linkplain Number}</li>
	 * <li>{@linkplain Boolean}</li>
	 * <li>{@code null}</li>
	 * </ul>
	 * @throws IOException        if an I/O error occurs
	 * @throws JSONParseException if the JSON is invalid
	 * @since 1.0.0
	 */
	public Object parse(Reader reader) throws IOException, JSONParseException {

		// ==== 11.03.2018 | Ralph Niemitz/RalleYTN(ralph.niemitz@gmx.de)
		// -	Removed the unnecessary try-catch block
		// ====

		this.reset(reader);
		final Stack<Object> statusStack = new Stack<>();
		final Stack<Object> valueStack = new Stack<>();

		do {

			this.nextToken();

			if (this.status == JSONParser.S_INIT)
				this.init(statusStack, valueStack);
			else if (this.status == JSONParser.S_IN_FINISHED_VALUE)
				return this.inFinishedValue(valueStack);
			else if (this.status == JSONParser.S_IN_OBJECT)
				this.inObject(statusStack, valueStack);
			else if (this.status == JSONParser.S_PASSED_PAIR_KEY)
				this.inPassedPairKey(statusStack, valueStack);
			else if (this.status == JSONParser.S_IN_ARRAY)
				this.inArray(statusStack, valueStack);

			if (this.status == JSONParser.S_IN_ERROR)
				throw new JSONParseException(getPosition(), JSONParseException.ERROR_UNEXPECTED_TOKEN, token);

		} while (this.token.type != Yytoken.TYPE_EOF);

		throw new JSONParseException(this.getPosition(), JSONParseException.ERROR_UNEXPECTED_TOKEN, this.token);
	}

	/**
	 * Goes over a JSON string step by step using a {@linkplain JSONContentHandler}.
	 *
	 * @param string         the JSON string
	 * @param contentHandler the {@linkplain JSONContentHandler}
	 * @throws JSONParseException if the JSON is invalid or the {@linkplain JSONContentHandler} throws it
	 * @since 1.0.0
	 */
	public void parse(String string, JSONContentHandler contentHandler) throws JSONParseException {

		this.parse(string != null ? string.trim() : null, contentHandler, false);
	}

	/**
	 * Goes over a JSON string step by step using a {@linkplain JSONContentHandler}.
	 *
	 * @param string         the JSON string
	 * @param contentHandler the {@linkplain JSONContentHandler}
	 * @param resume         Indicates if the previous parsing operation should be continued.
	 * @throws JSONParseException if the JSON is invalid or the {@linkplain JSONContentHandler} throws it
	 * @since 1.0.0
	 */
	public void parse(String string, JSONContentHandler contentHandler, boolean resume) throws JSONParseException {

		try (StringReader reader = new StringReader(string != null ? string.trim() : null)) {

			this.parse(reader, contentHandler, resume);

		} catch (final IOException exception) {

			throw new JSONParseException(-1, JSONParseException.ERROR_UNEXPECTED_EXCEPTION, exception);
		}
	}

	/**
	 * Goes over JSON data step by step using a {@linkplain JSONContentHandler}.
	 *
	 * @param reader         the {@linkplain Reader}
	 * @param contentHandler the {@linkplain JSONContentHandler}
	 * @throws IOException        if an I/O error occurs or the {@linkplain JSONContentHandler} throws it
	 * @throws JSONParseException if the JSON is invalid or the {@linkplain JSONContentHandler} throws it
	 * @since 1.0.0
	 */
	public void parse(Reader reader, JSONContentHandler contentHandler) throws IOException, JSONParseException {

		this.parse(reader, contentHandler, false);
	}

	/**
	 * Goes over JSON data step by step using a {@linkplain JSONContentHandler}.
	 *
	 * @param reader         the {@linkplain Reader}
	 * @param contentHandler the {@linkplain JSONContentHandler}
	 * @param resume         Indicates if the previous parsing operation should be continued.
	 * @throws IOException        if an I/O error occurs or the {@linkplain JSONContentHandler} throws it
	 * @throws JSONParseException if the JSON is invalid or the {@linkplain JSONContentHandler} throws it
	 * @see JSONContentHandler
	 * @since 1.0.0
	 */
	public void parse(Reader reader, JSONContentHandler contentHandler, boolean resume) throws IOException, JSONParseException {

		if (!resume) {

			this.reset(reader);
			this.handlerStatusStack = new Stack<>();

		} else if (this.handlerStatusStack == null) {

			this.reset(reader);
			this.handlerStatusStack = new Stack<>();
		}

		final Stack<Object> statusStack = this.handlerStatusStack;

		try {

			do {

				if (this.status == JSONParser.S_INIT) {

					contentHandler.startJSON();
					this.nextToken();

					if (this.token.type == Yytoken.TYPE_VALUE) {

						this.status = JSONParser.S_IN_FINISHED_VALUE;
						statusStack.push(this.status);

						if (!contentHandler.primitive(this.token.value))
							return;

					} else if (this.token.type == Yytoken.TYPE_LEFT_BRACE) {

						this.status = JSONParser.S_IN_OBJECT;
						statusStack.push(this.status);

						if (!contentHandler.startObject())
							return;

					} else if (this.token.type == Yytoken.TYPE_LEFT_SQUARE) {

						this.status = JSONParser.S_IN_ARRAY;
						statusStack.push(this.status);

						if (!contentHandler.startArray())
							return;

					} else
						this.status = JSONParser.S_IN_ERROR;

				} else if (this.status == JSONParser.S_IN_FINISHED_VALUE) {

					this.nextToken();

					if (this.token.type == Yytoken.TYPE_EOF) {

						contentHandler.endJSON();
						status = JSONParser.S_END;
						return;

					} else
						this.status = JSONParser.S_IN_ERROR;

				} else if (this.status == JSONParser.S_IN_OBJECT) {

					this.nextToken();

					if (this.token.type == Yytoken.TYPE_VALUE) {

						if (this.token.value instanceof String) {

							final String key = (String) this.token.value;
							this.status = JSONParser.S_PASSED_PAIR_KEY;
							statusStack.push(this.status);

							if (!contentHandler.startObjectEntry(key))
								return;

						} else
							this.status = JSONParser.S_IN_ERROR;

					} else if (this.token.type == Yytoken.TYPE_RIGHT_BRACE) {

						if (statusStack.size() > 1) {

							statusStack.pop();
							this.status = (int) statusStack.peek();

						} else
							this.status = JSONParser.S_IN_FINISHED_VALUE;

						if (!contentHandler.endObject())
							return;

					} else if (this.token.type != Yytoken.TYPE_COMMA)
						this.status = JSONParser.S_IN_ERROR;

				} else if (this.status == JSONParser.S_PASSED_PAIR_KEY) {

					this.nextToken();

					if (this.token.type == Yytoken.TYPE_VALUE) {

						statusStack.pop();
						this.status = (int) statusStack.peek();

						if (!contentHandler.primitive(this.token.value) || !contentHandler.endObjectEntry())
							return;

					} else if (this.token.type == Yytoken.TYPE_LEFT_SQUARE) {

						statusStack.pop();
						statusStack.push(JSONParser.S_IN_PAIR_VALUE);
						this.status = JSONParser.S_IN_ARRAY;
						statusStack.push(this.status);

						if (!contentHandler.startArray())
							return;

					} else if (this.token.type == Yytoken.TYPE_LEFT_BRACE) {

						statusStack.pop();
						statusStack.push(JSONParser.S_IN_PAIR_VALUE);
						this.status = JSONParser.S_IN_OBJECT;
						statusStack.push(this.status);

						if (!contentHandler.startObject())
							return;

					} else if (this.token.type != Yytoken.TYPE_COLON)
						this.status = JSONParser.S_IN_ERROR;

				} else if (this.status == JSONParser.S_IN_PAIR_VALUE) {

					statusStack.pop();
					this.status = (int) statusStack.peek();

					if (!contentHandler.endObjectEntry())
						return;

				} else if (this.status == JSONParser.S_IN_ARRAY) {

					this.nextToken();

					if (this.token.type == Yytoken.TYPE_VALUE) {

						if (!contentHandler.primitive(this.token.value))
							return;

					} else if (this.token.type == Yytoken.TYPE_RIGHT_SQUARE) {

						if (statusStack.size() > 1) {

							statusStack.pop();
							this.status = (int) statusStack.peek();

						} else
							this.status = JSONParser.S_IN_FINISHED_VALUE;

						if (!contentHandler.endArray())
							return;

					} else if (this.token.type == Yytoken.TYPE_LEFT_BRACE) {

						this.status = JSONParser.S_IN_OBJECT;
						statusStack.push(this.status);

						if (!contentHandler.startObject())
							return;

					} else if (this.token.type == Yytoken.TYPE_LEFT_SQUARE) {

						this.status = JSONParser.S_IN_ARRAY;
						statusStack.push(this.status);

						if (!contentHandler.startArray())
							return;

					} else if (this.token.type != Yytoken.TYPE_COMMA)
						this.status = JSONParser.S_IN_ERROR;

				} else if (this.status == JSONParser.S_END)
					return;

				if (this.status == JSONParser.S_IN_ERROR)
					throw new JSONParseException(this.getPosition(), JSONParseException.ERROR_UNEXPECTED_TOKEN, this.token);

			} while (this.token.type != Yytoken.TYPE_EOF);

		} catch (IOException | JSONParseException | RuntimeException | Error exception) {

			this.status = JSONParser.S_IN_ERROR;
			throw exception;
		}

		this.status = JSONParser.S_IN_ERROR;
		throw new JSONParseException(this.getPosition(), JSONParseException.ERROR_UNEXPECTED_TOKEN, this.token);
	}
}
