/*
 *                                  Apache License
 *                            Version 2.0, January 2004
 *                         http://www.apache.org/licenses/
 *
 *    TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION
 *
 *    1. Definitions.
 *
 *       "License" shall mean the terms and conditions for use, reproduction,
 *       and distribution as defined by Sections 1 through 9 of this document.
 *
 *       "Licensor" shall mean the copyright owner or entity authorized by
 *       the copyright owner that is granting the License.
 *
 *       "Legal Entity" shall mean the union of the acting entity and all
 *       other entities that control, are controlled by, or are under common
 *       control with that entity. For the purposes of this definition,
 *       "control" means (i) the power, direct or indirect, to cause the
 *       direction or management of such entity, whether by contract or
 *       otherwise, or (ii) ownership of fifty percent (50%) or more of the
 *       outstanding shares, or (iii) beneficial ownership of such entity.
 *
 *       "You" (or "Your") shall mean an individual or Legal Entity
 *       exercising permissions granted by this License.
 *
 *       "Source" form shall mean the preferred form for making modifications,
 *       including but not limited to software source code, documentation
 *       source, and configuration files.
 *
 *       "Object" form shall mean any form resulting from mechanical
 *       transformation or translation of a Source form, including but
 *       not limited to compiled object code, generated documentation,
 *       and conversions to other media types.
 *
 *       "Work" shall mean the work of authorship, whether in Source or
 *       Object form, made available under the License, as indicated by a
 *       copyright notice that is included in or attached to the work
 *       (an example is provided in the Appendix below).
 *
 *       "Derivative Works" shall mean any work, whether in Source or Object
 *       form, that is based on (or derived from) the Work and for which the
 *       editorial revisions, annotations, elaborations, or other modifications
 *       represent, as a whole, an original work of authorship. For the purposes
 *       of this License, Derivative Works shall not include works that remain
 *       separable from, or merely link (or bind by name) to the interfaces of,
 *       the Work and Derivative Works thereof.
 *
 *       "Contribution" shall mean any work of authorship, including
 *       the original version of the Work and any modifications or additions
 *       to that Work or Derivative Works thereof, that is intentionally
 *       submitted to Licensor for inclusion in the Work by the copyright owner
 *       or by an individual or Legal Entity authorized to submit on behalf of
 *       the copyright owner. For the purposes of this definition, "submitted"
 *       means any form of electronic, verbal, or written communication sent
 *       to the Licensor or its representatives, including but not limited to
 *       communication on electronic mailing lists, source code control systems,
 *       and issue tracking systems that are managed by, or on behalf of, the
 *       Licensor for the purpose of discussing and improving the Work, but
 *       excluding communication that is conspicuously marked or otherwise
 *       designated in writing by the copyright owner as "Not a Contribution."
 *
 *       "Contributor" shall mean Licensor and any individual or Legal Entity
 *       on behalf of whom a Contribution has been received by Licensor and
 *       subsequently incorporated within the Work.
 *
 *    2. Grant of Copyright License. Subject to the terms and conditions of
 *       this License, each Contributor hereby grants to You a perpetual,
 *       worldwide, non-exclusive, no-charge, royalty-free, irrevocable
 *       copyright license to reproduce, prepare Derivative Works of,
 *       publicly display, publicly perform, sublicense, and distribute the
 *       Work and such Derivative Works in Source or Object form.
 *
 *    3. Grant of Patent License. Subject to the terms and conditions of
 *       this License, each Contributor hereby grants to You a perpetual,
 *       worldwide, non-exclusive, no-charge, royalty-free, irrevocable
 *       (except as stated in this section) patent license to make, have made,
 *       use, offer to sell, sell, import, and otherwise transfer the Work,
 *       where such license applies only to those patent claims licensable
 *       by such Contributor that are necessarily infringed by their
 *       Contribution(s) alone or by combination of their Contribution(s)
 *       with the Work to which such Contribution(s) was submitted. If You
 *       institute patent litigation against any entity (including a
 *       cross-claim or counterclaim in a lawsuit) alleging that the Work
 *       or a Contribution incorporated within the Work constitutes direct
 *       or contributory patent infringement, then any patent licenses
 *       granted to You under this License for that Work shall terminate
 *       as of the date such litigation is filed.
 *
 *    4. Redistribution. You may reproduce and distribute copies of the
 *       Work or Derivative Works thereof in any medium, with or without
 *       modifications, and in Source or Object form, provided that You
 *       meet the following conditions:
 *
 *       (a) You must give any other recipients of the Work or
 *           Derivative Works a copy of this License; and
 *
 *       (b) You must cause any modified files to carry prominent notices
 *           stating that You changed the files; and
 *
 *       (c) You must retain, in the Source form of any Derivative Works
 *           that You distribute, all copyright, patent, trademark, and
 *           attribution notices from the Source form of the Work,
 *           excluding those notices that do not pertain to any part of
 *           the Derivative Works; and
 *
 *       (d) If the Work includes a "NOTICE" text file as part of its
 *           distribution, then any Derivative Works that You distribute must
 *           include a readable copy of the attribution notices contained
 *           within such NOTICE file, excluding those notices that do not
 *           pertain to any part of the Derivative Works, in at least one
 *           of the following places: within a NOTICE text file distributed
 *           as part of the Derivative Works; within the Source form or
 *           documentation, if provided along with the Derivative Works; or,
 *           within a display generated by the Derivative Works, if and
 *           wherever such third-party notices normally appear. The contents
 *           of the NOTICE file are for informational purposes only and
 *           do not modify the License. You may add Your own attribution
 *           notices within Derivative Works that You distribute, alongside
 *           or as an addendum to the NOTICE text from the Work, provided
 *           that such additional attribution notices cannot be construed
 *           as modifying the License.
 *
 *       You may add Your own copyright statement to Your modifications and
 *       may provide additional or different license terms and conditions
 *       for use, reproduction, or distribution of Your modifications, or
 *       for any such Derivative Works as a whole, provided Your use,
 *       reproduction, and distribution of the Work otherwise complies with
 *       the conditions stated in this License.
 *
 *    5. Submission of Contributions. Unless You explicitly state otherwise,
 *       any Contribution intentionally submitted for inclusion in the Work
 *       by You to the Licensor shall be under the terms and conditions of
 *       this License, without any additional terms or conditions.
 *       Notwithstanding the above, nothing herein shall supersede or modify
 *       the terms of any separate license agreement you may have executed
 *       with Licensor regarding such Contributions.
 *
 *    6. Trademarks. This License does not grant permission to use the trade
 *       names, trademarks, service marks, or product names of the Licensor,
 *       except as required for reasonable and customary use in describing the
 *       origin of the Work and reproducing the content of the NOTICE file.
 *
 *    7. Disclaimer of Warranty. Unless required by applicable law or
 *       agreed to in writing, Licensor provides the Work (and each
 *       Contributor provides its Contributions) on an "AS IS" BASIS,
 *       WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 *       implied, including, without limitation, any warranties or conditions
 *       of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A
 *       PARTICULAR PURPOSE. You are solely responsible for determining the
 *       appropriateness of using or redistributing the Work and assume any
 *       risks associated with Your exercise of permissions under this License.
 *
 *    8. Limitation of Liability. In no event and under no legal theory,
 *       whether in tort (including negligence), contract, or otherwise,
 *       unless required by applicable law (such as deliberate and grossly
 *       negligent acts) or agreed to in writing, shall any Contributor be
 *       liable to You for damages, including any direct, indirect, special,
 *       incidental, or consequential damages of any character arising as a
 *       result of this License or out of the use or inability to use the
 *       Work (including but not limited to damages for loss of goodwill,
 *       work stoppage, computer failure or malfunction, or any and all
 *       other commercial damages or losses), even if such Contributor
 *       has been advised of the possibility of such damages.
 *
 *    9. Accepting Warranty or Additional Liability. While redistributing
 *       the Work or Derivative Works thereof, You may choose to offer,
 *       and charge a fee for, acceptance of support, warranty, indemnity,
 *       or other liability obligations and/or rights consistent with this
 *       License. However, in accepting such obligations, You may act only
 *       on Your own behalf and on Your sole responsibility, not on behalf
 *       of any other Contributor, and only if You agree to indemnify,
 *       defend, and hold each Contributor harmless for any liability
 *       incurred by, or claims asserted against, such Contributor by reason
 *       of your accepting any such warranty or additional liability.
 *
 *    END OF TERMS AND CONDITIONS
 *
 *    APPENDIX: How to apply the Apache License to your work.
 *
 *       To apply the Apache License to your work, attach the following
 *       boilerplate notice, with the fields enclosed by brackets "{}"
 *       replaced with your own identifying information. (Don't include
 *       the brackets!)  The text should be enclosed in the appropriate
 *       comment syntax for the file format. We also recommend that a
 *       file or class name and description of purpose be included on the
 *       same "printed page" as the copyright notice for easier
 *       identification within third-party archives.
 *
 *    Copyright {yyyy} {name of copyright owner}
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.mineacademy.fo.jsonsimple;

import java.lang.reflect.Array;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

import org.mineacademy.fo.jsonsimple.annotation.JSONAttribute;
import org.mineacademy.fo.jsonsimple.annotation.JSONRoot;

/**
 * Provides static methods which allow you to convert Java objects into {@linkplain JSONObject}s
 * and the other way around.
 *
 * @author Ralph Niemitz/RalleYTN(ralph.niemitz@gmx.de)
 * @version 2.0.0
 * @since 1.0.0
 */
public final class JSONSerializer {

	private JSONSerializer() {
	}

	/**
	 * Transfers the data of {@linkplain JSONObject} into a Java object.
	 *
	 * @param json   the JSON object containing the data
	 * @param object the body of the Java object that should be filled
	 * @throws Exception if an error occurs
	 * @since 1.0.0
	 */
	public static void deserialize(JSONObject json, Object object) throws Exception {

		final Class<?> clazz = object.getClass();
		final Field[] fields = clazz.getFields();
		final Method[] methods = clazz.getMethods();

		for (final Field field : fields) {

			final JSONAttribute annotation = field.getAnnotation(JSONAttribute.class);

			if (!Modifier.isTransient(field.getModifiers()) && annotation != null && JSONSerializer.contains(JSONAttribute.Type.SETTER, annotation)) {

				final Class<?> targetType = field.getType();
				final Object value = json.get(annotation.name());

				if (value != null)
					if (float.class.isAssignableFrom(targetType) || Float.class.isAssignableFrom(targetType))
						field.set(object, ((Number) value).floatValue());
					else if (double.class.isAssignableFrom(targetType) || Double.class.isAssignableFrom(targetType))
						field.set(object, ((Number) value).doubleValue());
					else if (byte.class.isAssignableFrom(targetType) || Byte.class.isAssignableFrom(targetType))
						field.set(object, ((Number) value).byteValue());
					else if (short.class.isAssignableFrom(targetType) || Short.class.isAssignableFrom(targetType))
						field.set(object, ((Number) value).shortValue());
					else if (int.class.isAssignableFrom(targetType) || Integer.class.isAssignableFrom(targetType))
						field.set(object, ((Number) value).intValue());
					else if (long.class.isAssignableFrom(targetType) || Long.class.isAssignableFrom(targetType))
						field.set(object, ((Number) value).longValue());
					else if (String.class.isAssignableFrom(targetType) || Number.class.isAssignableFrom(targetType) || Boolean.class.isAssignableFrom(targetType) || boolean.class.isAssignableFrom(targetType))
						field.set(object, value);
					else if (Enum.class.isAssignableFrom(targetType)) {

						for (final Object constant : targetType.getEnumConstants())
							if (((Enum<?>) constant).name().equals(value.toString())) {

								field.set(object, constant);
								break;
							}

					} else if (targetType.getAnnotation(JSONRoot.class) != null) {

						final Object newObject = targetType.getDeclaredConstructor().newInstance();
						JSONSerializer.deserialize((JSONObject) value, newObject);
						field.set(object, newObject);

					} else if (Map.class.isAssignableFrom(targetType)) {

						final Map<Object, Object> map = (Map<Object, Object>) targetType.getDeclaredConstructor().newInstance();
						JSONSerializer.deserialize(clazz, object, map, (JSONObject) value);
						field.set(object, map);

					} else if (Collection.class.isAssignableFrom(targetType)) {

						final Collection<Object> collection = (Collection<Object>) targetType.getDeclaredConstructor().newInstance();
						JSONSerializer.deserialize(clazz, object, collection, (JSONArray) value);
						field.set(object, collection);

					} else if (targetType.isArray()) {

						final Object newArray = Array.newInstance(targetType.getComponentType(), ((JSONArray) value).size());
						JSONSerializer.deserialize(clazz, object, newArray, (JSONArray) value);
						field.set(object, newArray);

					} else if (JSONTypeSerializationHandler.class.isAssignableFrom(clazz))
						field.set(object, ((JSONTypeSerializationHandler) object).deserialize(targetType, value));
			}
		}

		for (final Method method : methods) {

			final JSONAttribute annotation = method.getAnnotation(JSONAttribute.class);

			if (!Modifier.isTransient(method.getModifiers()) && annotation != null && JSONSerializer.contains(JSONAttribute.Type.SETTER, annotation)) {

				final Class<?> targetType = method.getParameterTypes()[0];

				if (targetType.isInterface())
					throw new Exception("Cannot deserialize an interface! Method: " + method.getName() + ", Interface: " + targetType.getName());

				final Object value = json.get(annotation.name());

				if (value != null)
					if (float.class.isAssignableFrom(targetType) || Float.class.isAssignableFrom(targetType))
						method.invoke(object, ((Number) value).floatValue());
					else if (double.class.isAssignableFrom(targetType) || Double.class.isAssignableFrom(targetType))
						method.invoke(object, ((Number) value).doubleValue());
					else if (byte.class.isAssignableFrom(targetType) || Byte.class.isAssignableFrom(targetType))
						method.invoke(object, ((Number) value).byteValue());
					else if (short.class.isAssignableFrom(targetType) || Short.class.isAssignableFrom(targetType))
						method.invoke(object, ((Number) value).shortValue());
					else if (int.class.isAssignableFrom(targetType) || Integer.class.isAssignableFrom(targetType))
						method.invoke(object, ((Number) value).intValue());
					else if (long.class.isAssignableFrom(targetType) || Long.class.isAssignableFrom(targetType))
						method.invoke(object, ((Number) value).longValue());
					else if (String.class.isAssignableFrom(targetType) || Number.class.isAssignableFrom(targetType) || Boolean.class.isAssignableFrom(targetType) || boolean.class.isAssignableFrom(targetType))
						method.invoke(object, value);
					else if (Enum.class.isAssignableFrom(targetType)) {

						for (final Object constant : targetType.getEnumConstants())
							if (((Enum<?>) constant).name().equals(value.toString())) {

								method.invoke(object, constant);
								break;
							}

					} else if (targetType.getAnnotation(JSONRoot.class) != null) {

						final Object newObject = targetType.getDeclaredConstructor().newInstance();
						JSONSerializer.deserialize((JSONObject) value, newObject);
						method.invoke(object, newObject);

					} else if (Map.class.isAssignableFrom(targetType)) {

						final Map<Object, Object> map = (Map<Object, Object>) targetType.getDeclaredConstructor().newInstance();
						JSONSerializer.deserialize(clazz, object, map, (JSONObject) value);
						method.invoke(object, map);

					} else if (Collection.class.isAssignableFrom(targetType)) {

						final Collection<Object> collection = (Collection<Object>) targetType.getDeclaredConstructor().newInstance();
						JSONSerializer.deserialize(clazz, object, collection, (JSONArray) value);
						method.invoke(object, collection);

					} else if (targetType.isArray()) {

						final Object newArray = Array.newInstance(targetType.getComponentType(), ((JSONArray) value).size());
						JSONSerializer.deserialize(clazz, object, newArray, (JSONArray) value);
						method.invoke(object, newArray);

					} else if (JSONTypeSerializationHandler.class.isAssignableFrom(clazz))
						method.invoke(object, ((JSONTypeSerializationHandler) object).deserialize(targetType, value));
			}
		}
	}

	private static void deserialize(Class<?> clazz, Object classObject, Collection<Object> collection, JSONArray json) throws Exception {

		for (final Object value : json)
			if (value != null) {

				final Class<?> type = value.getClass();

				if (type.isInterface())
					throw new Exception("Cannot deserialize an interface! Interface: " + type.getName());

				if (float.class.isAssignableFrom(type) || Float.class.isAssignableFrom(type))
					collection.add(((Number) value).floatValue());
				else if (double.class.isAssignableFrom(type) || Double.class.isAssignableFrom(type))
					collection.add(((Number) value).doubleValue());
				else if (byte.class.isAssignableFrom(type) || Byte.class.isAssignableFrom(type))
					collection.add(((Number) value).byteValue());
				else if (short.class.isAssignableFrom(type) || Short.class.isAssignableFrom(type))
					collection.add(((Number) value).shortValue());
				else if (int.class.isAssignableFrom(type) || Integer.class.isAssignableFrom(type))
					collection.add(((Number) value).intValue());
				else if (long.class.isAssignableFrom(type) || Long.class.isAssignableFrom(type))
					collection.add(((Number) value).longValue());
				else if (String.class.isAssignableFrom(type) || Number.class.isAssignableFrom(type) || Boolean.class.isAssignableFrom(type) || boolean.class.isAssignableFrom(type))
					collection.add(value);
				else if (Enum.class.isAssignableFrom(type)) {

					for (final Object constant : type.getEnumConstants())
						if (((Enum<?>) constant).name().equals(value.toString())) {

							collection.add(constant);
							break;
						}

				} else if (type.getAnnotation(JSONRoot.class) != null) {

					final Object newObject = type.getDeclaredConstructor().newInstance();
					JSONSerializer.deserialize((JSONObject) value, newObject);
					collection.add(newObject);

				} else if (Map.class.isAssignableFrom(type)) {

					final Map<Object, Object> newMap = (Map<Object, Object>) type.getDeclaredConstructor().newInstance();
					JSONSerializer.deserialize(clazz, classObject, newMap, (JSONObject) value);
					collection.add(newMap);

				} else if (Collection.class.isAssignableFrom(type)) {

					final Collection<Object> newCollection = (Collection<Object>) type.getDeclaredConstructor().newInstance();
					JSONSerializer.deserialize(clazz, classObject, newCollection, (JSONArray) value);
					collection.add(newCollection);

				} else if (type.isArray()) {

					final Object newArray = Array.newInstance(type.getComponentType(), ((JSONArray) value).size());
					JSONSerializer.deserialize(clazz, classObject, newArray, (JSONArray) value);
					collection.add(newArray);

				} else if (JSONTypeSerializationHandler.class.isAssignableFrom(clazz))
					collection.add(((JSONTypeSerializationHandler) classObject).deserialize(type, value));

			} else
				collection.add(value);
	}

	private static void deserialize(Class<?> clazz, Object classObject, Map<Object, Object> map, JSONObject json) throws Exception {

		for (final Object jsonEntry : json.entrySet()) {

			final Map.Entry<?, ?> entry = (Map.Entry<?, ?>) jsonEntry;
			final Object value = entry.getValue();

			if (value != null) {

				final Class<?> type = value.getClass();

				if (type.isInterface())
					throw new Exception("Cannot deserialize an interface! Interface: " + type.getName());

				if (float.class.isAssignableFrom(type) || Float.class.isAssignableFrom(type))
					map.put(entry.getKey(), ((Number) value).floatValue());
				else if (double.class.isAssignableFrom(type) || Double.class.isAssignableFrom(type))
					map.put(entry.getKey(), ((Number) value).doubleValue());
				else if (byte.class.isAssignableFrom(type) || Byte.class.isAssignableFrom(type))
					map.put(entry.getKey(), ((Number) value).byteValue());
				else if (short.class.isAssignableFrom(type) || Short.class.isAssignableFrom(type))
					map.put(entry.getKey(), ((Number) value).shortValue());
				else if (int.class.isAssignableFrom(type) || Integer.class.isAssignableFrom(type))
					map.put(entry.getKey(), ((Number) value).intValue());
				else if (long.class.isAssignableFrom(type) || Long.class.isAssignableFrom(type))
					map.put(entry.getKey(), ((Number) value).longValue());
				else if (String.class.isAssignableFrom(type) || Number.class.isAssignableFrom(type) || Boolean.class.isAssignableFrom(type) || boolean.class.isAssignableFrom(type))
					map.put(entry.getKey(), value);
				else if (Enum.class.isAssignableFrom(type)) {

					for (final Object constant : type.getEnumConstants())
						if (((Enum<?>) constant).name().equals(value.toString())) {

							map.put(entry.getKey(), constant);
							break;
						}

				} else if (type.getAnnotation(JSONRoot.class) != null) {

					final Object newObject = type.getDeclaredConstructor().newInstance();
					JSONSerializer.deserialize((JSONObject) value, newObject);
					map.put(entry.getKey(), newObject);

				} else if (Map.class.isAssignableFrom(type)) {

					final Map<Object, Object> newMap = (Map<Object, Object>) type.getDeclaredConstructor().newInstance();
					JSONSerializer.deserialize(clazz, classObject, newMap, (JSONObject) value);
					map.put(entry.getKey(), newMap);

				} else if (Collection.class.isAssignableFrom(type)) {

					final Collection<Object> collection = (Collection<Object>) type.getDeclaredConstructor().newInstance();
					JSONSerializer.deserialize(clazz, classObject, collection, (JSONArray) value);
					map.put(entry.getKey(), collection);

				} else if (type.isArray()) {

					final Object newArray = Array.newInstance(type.getComponentType(), ((JSONArray) value).size());
					JSONSerializer.deserialize(clazz, classObject, newArray, (JSONArray) value);
					map.put(entry.getKey(), newArray);

				} else if (JSONTypeSerializationHandler.class.isAssignableFrom(clazz))
					map.put(entry.getKey(), ((JSONTypeSerializationHandler) classObject).deserialize(type, value));

			} else
				map.put(entry.getKey(), value);
		}
	}

	private static void deserialize(Class<?> clazz, Object classObject, Object targetObject, JSONArray json) throws Exception {

		final Class<?> type = targetObject.getClass().getComponentType();

		if (type.isInterface())
			throw new Exception("Cannot deserialize an interface! Interface: " + type.getName());

		for (int index = 0; index < json.size(); index++) {

			final Object value = json.get(index);

			if (float.class.isAssignableFrom(type) || Float.class.isAssignableFrom(type))
				Array.set(targetObject, index, ((Number) value).floatValue());
			else if (double.class.isAssignableFrom(type) || Double.class.isAssignableFrom(type))
				Array.set(targetObject, index, ((Number) value).doubleValue());
			else if (byte.class.isAssignableFrom(type) || Byte.class.isAssignableFrom(type))
				Array.set(targetObject, index, ((Number) value).byteValue());
			else if (short.class.isAssignableFrom(type) || Short.class.isAssignableFrom(type))
				Array.set(targetObject, index, ((Number) value).shortValue());
			else if (int.class.isAssignableFrom(type) || Integer.class.isAssignableFrom(type))
				Array.set(targetObject, index, ((Number) value).intValue());
			else if (long.class.isAssignableFrom(type) || Long.class.isAssignableFrom(type))
				Array.set(targetObject, index, ((Number) value).longValue());
			else if (String.class.isAssignableFrom(type) || Number.class.isAssignableFrom(type) || Boolean.class.isAssignableFrom(type) || boolean.class.isAssignableFrom(type))
				Array.set(targetObject, index, value);
			else if (Enum.class.isAssignableFrom(type)) {

				for (final Object constant : type.getEnumConstants())
					if (((Enum<?>) constant).name().equals(value.toString())) {

						Array.set(targetObject, index, constant);
						break;
					}

			} else if (type.getAnnotation(JSONRoot.class) != null) {

				final Object newObject = type.getDeclaredConstructor().newInstance();
				JSONSerializer.deserialize((JSONObject) value, newObject);
				Array.set(targetObject, index, newObject);

			} else if (Map.class.isAssignableFrom(type)) {

				final Map<Object, Object> map = (Map<Object, Object>) type.getDeclaredConstructor().newInstance();
				JSONSerializer.deserialize(clazz, classObject, map, (JSONObject) value);
				Array.set(targetObject, index, map);

			} else if (Collection.class.isAssignableFrom(type)) {

				final Collection<Object> collection = (Collection<Object>) type.getDeclaredConstructor().newInstance();
				JSONSerializer.deserialize(clazz, classObject, collection, (JSONArray) value);
				Array.set(targetObject, index, collection);

			} else if (type.isArray()) {

				final Object newArray = Array.newInstance(type.getComponentType(), ((JSONArray) value).size());
				JSONSerializer.deserialize(clazz, classObject, newArray, (JSONArray) value);
				Array.set(targetObject, index, newArray);

			} else if (JSONTypeSerializationHandler.class.isAssignableFrom(clazz))
				Array.set(targetObject, index, ((JSONTypeSerializationHandler) classObject).deserialize(type, value));
		}
	}

	/**
	 * Converts a Java object into a {@linkplain JSONObject}.
	 *
	 * @param object the Java object you want to convert
	 * @return the resulting {@linkplain JSONObject}
	 * @throws Exception if an error occurs
	 * @since 1.0.0
	 */
	public static JSONObject serialize(Object object) throws Exception {

		final Class<?> clazz = object.getClass();
		final Field[] fields = clazz.getFields();
		final Method[] methods = clazz.getMethods();
		final JSONObject json = new JSONObject();

		for (final Field field : fields) {

			final JSONAttribute annotation = field.getAnnotation(JSONAttribute.class);

			if (!Modifier.isTransient(field.getModifiers()) && annotation != null && JSONSerializer.contains(JSONAttribute.Type.GETTER, annotation))
				JSONSerializer.serialize(json, clazz, object, field.getType(), field.get(object), annotation);
		}

		for (final Method method : methods) {

			final JSONAttribute annotation = method.getAnnotation(JSONAttribute.class);

			if (!Modifier.isTransient(method.getModifiers()) && annotation != null && JSONSerializer.contains(JSONAttribute.Type.GETTER, annotation))
				JSONSerializer.serialize(json, clazz, object, method.getReturnType(), method.invoke(object), annotation);
		}

		return json;
	}

	private static void serialize(JSONObject json, Class<?> clazz, Object classObject, Class<?> type, Object value, JSONAttribute annotation) throws Exception {

		// ==== 17.03.2018 | Ralph Niemitz/RalleYTN(ralph.niemitz@gmx.de)
		// -	Numbers now always get converted to longs
		// ====

		if (value instanceof String || value instanceof Boolean)
			json.put(annotation.name(), value);
		else if (value instanceof Number)
			json.put(annotation.name(), value != null ? ((Number) value).longValue() : null);
		else if (value instanceof boolean[])
			json.put(annotation.name(), new JSONArray((boolean[]) value));
		else if (value instanceof byte[])
			json.put(annotation.name(), new JSONArray((byte[]) value));
		else if (value instanceof short[])
			json.put(annotation.name(), new JSONArray((short[]) value));
		else if (value instanceof int[])
			json.put(annotation.name(), new JSONArray((int[]) value));
		else if (value instanceof long[])
			json.put(annotation.name(), new JSONArray((long[]) value));
		else if (value instanceof float[])
			json.put(annotation.name(), new JSONArray((float[]) value));
		else if (value instanceof double[])
			json.put(annotation.name(), new JSONArray((double[]) value));
		else if (value != null && value.getClass().isArray())
			json.put(annotation.name(), JSONSerializer.serializeArray(value, clazz, classObject));
		else if (value != null && value.getClass().getAnnotation(JSONRoot.class) != null)
			json.put(annotation.name(), JSONSerializer.serialize(value));
		else if (value instanceof Collection)
			json.put(annotation.name(), JSONSerializer.serializeArray((Collection<?>) value, clazz, classObject));
		else if (value instanceof Map)
			json.put(annotation.name(), JSONSerializer.serializeObject((Map<?, ?>) value, clazz, classObject));
		else if (value instanceof Enum)
			json.put(annotation.name(), ((Enum<?>) value).name());
		else if (JSONTypeSerializationHandler.class.isAssignableFrom(clazz))
			json.put(annotation.name(), ((JSONTypeSerializationHandler) classObject).serialize(type, value));
		else
			json.put(annotation.name(), value);
	}

	private static JSONObject serializeObject(Map<?, ?> map, Class<?> clazz, Object classObject) throws Exception {

		final JSONObject json = new JSONObject();

		for (final Map.Entry<?, ?> entry : map.entrySet()) {

			final Object value = entry.getValue();

			if (value instanceof boolean[])
				json.put(entry.getKey(), new JSONArray((boolean[]) value));
			else if (value instanceof byte[])
				json.put(entry.getKey(), new JSONArray((byte[]) value));
			else if (value instanceof short[])
				json.put(entry.getKey(), new JSONArray((short[]) value));
			else if (value instanceof int[])
				json.put(entry.getKey(), new JSONArray((int[]) value));
			else if (value instanceof long[])
				json.put(entry.getKey(), new JSONArray((long[]) value));
			else if (value instanceof float[])
				json.put(entry.getKey(), new JSONArray((float[]) value));
			else if (value instanceof double[])
				json.put(entry.getKey(), new JSONArray((double[]) value));
			else if (value != null && value.getClass().getAnnotation(JSONRoot.class) != null)
				json.put(entry.getKey(), JSONSerializer.serialize(value));
			else if (value instanceof Map)
				json.put(entry.getKey(), JSONSerializer.serializeObject((Map<?, ?>) value, clazz, classObject));
			else if (value instanceof Collection)
				json.put(entry.getKey(), JSONSerializer.serializeArray((Collection<?>) value, clazz, classObject));
			else if (value != null && value.getClass().isArray())
				json.put(entry.getKey(), JSONSerializer.serializeArray(value, clazz, classObject));
			else if (value instanceof Enum)
				json.put(entry.getKey(), ((Enum<?>) value).name());
			else if (value instanceof String || value instanceof Boolean || value instanceof Number)
				json.put(entry.getKey(), value);
			else if (value != null && JSONTypeSerializationHandler.class.isAssignableFrom(clazz))
				json.put(entry.getKey(), ((JSONTypeSerializationHandler) classObject).serialize(value.getClass(), value));
			else
				json.put(entry.getKey(), value);
		}

		return json;
	}

	private static JSONArray serializeArray(Collection<?> collection, Class<?> clazz, Object classObject) throws Exception {

		final JSONArray json = new JSONArray();
		final Iterator<?> iterator = collection.iterator();

		while (iterator.hasNext()) {

			final Object value = iterator.next();

			if (value instanceof boolean[])
				json.add(new JSONArray((boolean[]) value));
			else if (value instanceof byte[])
				json.add(new JSONArray((byte[]) value));
			else if (value instanceof short[])
				json.add(new JSONArray((short[]) value));
			else if (value instanceof int[])
				json.add(new JSONArray((int[]) value));
			else if (value instanceof long[])
				json.add(new JSONArray((long[]) value));
			else if (value instanceof float[])
				json.add(new JSONArray((float[]) value));
			else if (value instanceof double[])
				json.add(new JSONArray((double[]) value));
			else if (value != null && value.getClass().getAnnotation(JSONRoot.class) != null)
				json.add(JSONSerializer.serialize(value));
			else if (value instanceof Map)
				json.add(JSONSerializer.serializeObject((Map<?, ?>) value, clazz, classObject));
			else if (value instanceof Collection)
				json.add(JSONSerializer.serializeArray((Collection<?>) value, clazz, classObject));
			else if (value != null && value.getClass().isArray())
				json.add(JSONSerializer.serializeArray(value, clazz, classObject));
			else if (value instanceof Enum)
				json.add(((Enum<?>) value).name());
			else if (value instanceof String || value instanceof Boolean || value instanceof Number)
				json.add(value);
			else if (value != null && JSONTypeSerializationHandler.class.isAssignableFrom(clazz))
				((JSONTypeSerializationHandler) classObject).serialize(value.getClass(), value);
			else
				json.add(value);
		}

		return json;
	}

	private static JSONArray serializeArray(Object array, Class<?> clazz, Object classObject) throws Exception {

		final JSONArray json = new JSONArray();

		for (int index = 0; index < Array.getLength(array); index++) {

			final Object value = Array.get(array, index);

			if (value instanceof boolean[])
				json.add(new JSONArray((boolean[]) value));
			else if (value instanceof byte[])
				json.add(new JSONArray((byte[]) value));
			else if (value instanceof short[])
				json.add(new JSONArray((short[]) value));
			else if (value instanceof int[])
				json.add(new JSONArray((int[]) value));
			else if (value instanceof long[])
				json.add(new JSONArray((long[]) value));
			else if (value instanceof float[])
				json.add(new JSONArray((float[]) value));
			else if (value instanceof double[])
				json.add(new JSONArray((double[]) value));
			else if (value != null && value.getClass().getAnnotation(JSONRoot.class) != null)
				json.add(JSONSerializer.serialize(value));
			else if (value instanceof Map)
				json.add(JSONSerializer.serializeObject((Map<?, ?>) value, clazz, classObject));
			else if (value instanceof Collection)
				json.add(JSONSerializer.serializeArray((Collection<?>) value, clazz, classObject));
			else if (value != null && value.getClass().isArray())
				json.add(JSONSerializer.serializeArray(value, clazz, classObject));
			else if (value instanceof Enum)
				json.add(((Enum<?>) value).name());
			else if (value instanceof String || value instanceof Boolean || value instanceof Number)
				json.add(value);
			else if (value != null && JSONTypeSerializationHandler.class.isAssignableFrom(clazz))
				json.add(((JSONTypeSerializationHandler) classObject).serialize(value.getClass(), value));
			else
				json.add(value);
		}

		return json;
	}

	private static boolean contains(JSONAttribute.Type type, JSONAttribute attribute) {

		for (final JSONAttribute.Type value : attribute.type())
			if (value.equals(type))
				return true;

		return false;
	}
}
